// naumann@stce.rwth-aachen.de

#include <iostream> 
#include <fstream> 
#include <cassert> 
#include <cmath> 
#include <vector> 
#include <limits>

template<typename T>
T model(T p, T x) {
  return pow(p*x,2);
}

template<typename T>
void read_data(std::ifstream& in, T &p, std::vector<T> &x, std::vector<T> &y) {
  in >> p; size_t m=0; in >> m; 
  for (size_t i=0;i<m;i++) {
    T aux; 
    in >> aux; x.push_back(aux);
    in >> aux; y.push_back(aux);
  }
}

template<typename T>
void write_data(std::ofstream& out, const T &p, const std::vector<T> &x, const std::vector<T> &y) {
  assert(x.size()==y.size());
  out << p << std::endl << x.size() << std::endl; 
  for (size_t i=0;i<x.size();i++) 
    out << x[i] << " " << y[i] << std::endl;
}

template<typename T>
T error(T p, const std::vector<T> &x, const std::vector<T> &y) {
  assert(x.size()==y.size());
  T e=0;
  for (size_t i=0;i<x.size();i++) 
    e+=pow(model(p,x[i])-y[i],2);
  return e;
}

template<typename T>
T d_error(T p, const std::vector<T> &x, const std::vector<T> &y) {
  const T eps=std::numeric_limits<T>::epsilon();
  const T delta_p=(p!=0) ? sqrt(sqrt(eps))*fabs(p) : sqrt(sqrt(eps));
  return (error(p+delta_p,x,y)-error(p,x,y))/delta_p;
}

template<typename T>
T dd_error(T p, const std::vector<T> &x, const std::vector<T> &y) {
  const T eps=std::numeric_limits<T>::epsilon();
  const T delta_p=(p!=0) ? sqrt(sqrt(eps))*fabs(p) : sqrt(sqrt(eps));
  return (d_error(p+delta_p,x,y)-d_error(p,x,y))/delta_p;
}

template<typename T>
T estimate(T p, const std::vector<T> &x, const std::vector<T> &y, T eps) {
  while (fabs(d_error(p,x,y))>eps) {
    T d=dd_error(p,x,y); assert(d!=0);
    p-=d_error(p,x,y)/d;
  }
  return p;
}

int main(int argc, char *argv[]) {
  using T=double;
  assert(argc==2); 

  std::ifstream in(argv[1]); 
  T p=0; std::vector<T> x,y; 
  read_data(in,p,x,y);
  in.close(); 

  std::cout << "new observation:" << std::endl;
  T aux;
  std::cout << "x="; std::cin >> aux; x.push_back(aux);
  std::cout << "y="; std::cin >> aux; y.push_back(aux);
  p=estimate(p,x,y,sqrt(std::numeric_limits<T>::epsilon()));
  std::cout << "new p=" << p << std::endl;

  std::ofstream out(argv[1]); 
  write_data(out,p,x,y);
  out.close(); 
  return 0;
}

