// naumann@stce.rwth-aachen.de

#include "model.hpp"

#include <cmath> 
#include <vector> 
#include <limits>

template<typename T>
T error(T p, const std::vector<T> &x, const std::vector<T> &y) {
  assert(x.size()==y.size());
  T e=0;
  for (size_t i=0;i<x.size();i++) 
    e+=pow(model(p,x[i])-y[i],2);
  return e;
}

template<typename T>
T d_error(T p, const std::vector<T> &x, const std::vector<T> &y) {
  const T eps=std::numeric_limits<T>::epsilon();
  const T delta_p=(p!=0) ? sqrt(sqrt(eps))*fabs(p) : sqrt(sqrt(eps));
  return (error(p+delta_p,x,y)-error(p,x,y))/delta_p;
}

template<typename T>
T dd_error(T p, const std::vector<T> &x, const std::vector<T> &y) {
  const T eps=std::numeric_limits<T>::epsilon();
  const T delta_p=(p!=0) ? sqrt(sqrt(eps))*fabs(p) : sqrt(sqrt(eps));
  return (d_error(p+delta_p,x,y)-d_error(p,x,y))/delta_p;
}

#include "model.cpp"
